------------------------------------------------------
-- GFWUtils.lua
-- Useful utility / debug functions
------------------------------------------------------

GFWUTILS_THIS_VERSION = 11;

------------------------------------------------------

-- Shortcuts for common text color codes
function GFWUtils_temp_HiliteText(text)
	if (text == nil) then return nil; end
	return HIGHLIGHT_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
end
function GFWUtils_temp_RedText(text)
	if (text == nil) then return nil; end
	return RED_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
end
function GFWUtils_temp_GrayText(text)
	if (text == nil) then return nil; end
	return GRAY_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
end
function GFWUtils_temp_LightYellowText(text)
	if (text == nil) then return nil; end
	return LIGHTYELLOW_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
end

-- Plays sound only on a new item
function GFWUtils_temp_PlaySound(itemNum, interval)
	if (FleecingTipDB.NoSound) then return; end;
	local Act = nil
	if (GFWUtils.PlaySoundCache == nil) then
		GFWUtils.PlaySoundCache = itemNum;
		Act = true;
	end
	if (GFWUtils.PlaySoundCache ~= itemNum) then
		GFWUtils.PlaySoundCache = itemNum;
		Act = true;
	end
	if (Act) then
		PlaySound(3580); -- sheep death
	end
	
--	if (GFWUtils.PlaySoundCache == nil) then
--		GFWUtils.PlaySoundCache = {};
--	end
--	if (interval == nil and GFWUtils.PlaySoundCache[itemNum]) then
--		return;
--	end
--	if (interval and GFWUtils.PlaySoundCache[itemNum] and GetTime() - GFWUtils.PlaySoundCache[itemNum] < interval) then
--		return;
--	end
--	PlaySound(3580); -- sheep death
--	GFWUtils.PlaySoundCache[itemNum] = GetTime();
end

-- Prints a message to the chat frame
function GFWUtils_temp_Print(message, r, g, b)
	DEFAULT_CHAT_FRAME:AddMessage(message, (r or GFW_FONT_COLOR.r), (g or GFW_FONT_COLOR.g), (b or GFW_FONT_COLOR.b));
end

-- Prints a message to the chat frame once per interval (seconds) or once per session if interval is nil
function GFWUtils_temp_PrintOnce(message, interval, r, g, b)
	if (GFWUtils.PrintOnceCache == nil) then
		GFWUtils.PrintOnceCache = {};
	end
	if (interval == nil and GFWUtils.PrintOnceCache[message]) then
		return;
	end
	if (interval and GFWUtils.PrintOnceCache[message] and GetTime() - GFWUtils.PrintOnceCache[message] < interval) then
		return;
	end
	DEFAULT_CHAT_FRAME:AddMessage(message, (r or GFW_FONT_COLOR.r), (g or GFW_FONT_COLOR.g), (b or GFW_FONT_COLOR.b));
	GFWUtils.PrintOnceCache[message] = GetTime();
end

-- Prints a message to the chat frame only if Debug is set
function GFWUtils_temp_DebugLog(message)
	if (GFWUtils.Debug) then
		DEFAULT_CHAT_FRAME:AddMessage(message, GFW_DEBUG_COLOR.r, GFW_DEBUG_COLOR.g, GFW_DEBUG_COLOR.b);
	end
end

-- Prints a message in yellow to the floating messages frame
function GFWUtils_temp_Note(message)
	UIErrorsFrame:AddMessage(message, 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
end

-- Converts string.format to a string.find pattern: "%s hits %s for %d." to "(.+) hits (.+) for (%d+)"
-- based on Recap by Gello
function GFWUtils_temp_FormatToPattern(formatString)

	local patternString = formatString;
	
	patternString = string.gsub(patternString, "%%%d+%$([diouXxfgbcsq])", "%%%1"); -- reordering specifiers (e.g. %2$s) stripped	
	patternString = string.gsub(patternString, "([%$%(%)%.%[%]%*%+%-%?%^])", "%%%1"); -- convert regex special characters
	
	patternString = string.gsub(patternString, "%%c", "(.)"); -- %c to (.)
	patternString = string.gsub(patternString, "%%s", "(.+)"); -- %s to (.+)
	patternString = string.gsub(patternString, "%%[du]", "(%%d+)"); -- %d to (%d+)
	patternString = string.gsub(patternString, "%%([gf])", "(%%d+%%.*%%d*)"); -- %g or %f to (%d+%.*%d*)
		
	return patternString;

end

-- Splits a string into a table of strings separated by 'separator'.
-- e.g. Split(aString, ", ") is the reverse of table.concat(aTable, ", ")
function GFWUtils_temp_Split(aString, separator)
	if (aString == nil) then return nil; end
	
	local t = {};
	local function helper(segment)
		table.insert(t, segment);
	end
	helper((string.gsub(aString, "(.-)"..separator, helper)));
	return t;
end

-- Capitalizes the first letter of each word in aString.
function GFWUtils_temp_TitleCase(aString)
	if (aString == nil) then return nil; end
	local function capWords(first, rest) 
		return string.upper(first)..string.lower(rest);
	end
	return string.gsub(aString, "(%w)([%w_']*)", capWords);
end

-- large number brake down
local function brakedownNumbers(amount)
--	amount = 1234567890 -- for testing purposes!!!!
    assert (type (amount) == "number" or type (amount) == "string")
    local result = ""
    local sign, before, after = string.match (tostring (amount), "^([%+%-]?)(%d*)(%.?.*)$")
    while string.len (before) > 3 do
        result = ","..string.sub (before, -3, -1)..result
        before = string.sub (before, 1, -4)
    end
    return sign..before..result..after
end

-- Splits a cash amount into gold, siver, and copper
function GFWUtils_temp_GSC(money)
	if (money == nil) then money = 0; end
	local g = math.floor(money / 10000);
	local s = math.floor((money - (g*10000)) / 100);
	local c = math.floor(money - (g*10000) - (s*100));
	return g,s,c;
end

-- Formats money text by color for gold, silver, copper
-- discards copper for amounts greater than 1g unless second arg evaluates true
-- based on Auctioneer
function GFWUtils_temp_TextGSC(money, noRound)
	if (not money) then return 0; end;
	local g, s, c = GFWUtils.GSC(money);
	local texS = 12
	
	local GSC_GOLD = string.format("\124TInterface\\MoneyFrame\\UI-GoldIcon:%d:%d:-0.1:-0.5\124t", texS, texS);
	local GSC_SILVER = string.format("\124TInterface\\MoneyFrame\\UI-SilverIcon:%d:%d:-0.1:-0.5\124t", texS, texS);
	local GSC_COPPER = string.format("\124TInterface\\MoneyFrame\\UI-CopperIcon:%d:%d:-0.1:-0.5\124t", texS, texS);
	local GSC_START="|cffffffff%s|r%s";
	local GSC_PART=" |cffffffff%s|r%s";
	
	if FleecingTipDB.CoinFormat == "short" then
		GSC_GOLD = "|cffffd700"..GOLD_AMOUNT_SYMBOL.."|r";
		GSC_SILVER = "|cffc0c0c0"..SILVER_AMOUNT_SYMBOL.."|r";
		GSC_COPPER = "|cffc87533"..COPPER_AMOUNT_SYMBOL.."|r";
	elseif FleecingTipDB.CoinFormat == "long" then
		GSC_GOLD = "|cffffd700"..FT.GOLD.."|r";
		GSC_SILVER = "|cffc0c0c0"..FT.SILVER.."|r";
		GSC_COPPER = "|cffc87533"..FT.COPPER.."|r";
	end
	
	local gsc = "";
	if (c > 0) then
		gsc = string.format(GSC_START, tostring(c), GSC_COPPER);
	elseif (s > 0) then
		gsc = string.format(GSC_START, tostring(s), GSC_SILVER);
		if (c > 0) then
			gsc = gsc..string.format(GSC_PART, tostring(c), GSC_COPPER);
		end
	elseif (g > 0) then
		gsc = string.format(GSC_START, brakedownNumbers(g), GSC_GOLD);     
		if (s > 0) then
			gsc = gsc..string.format(GSC_PART, tostring(s), GSC_SILVER);
			if (c > 0) then
				gsc = gsc..string.format(GSC_PART, tostring(c), GSC_COPPER);
			end
		else
			if (c > 0) then
				gsc = gsc..string.format(GSC_PART, tostring(s), GSC_SILVER);
				gsc = gsc..string.format(GSC_PART, tostring(c), GSC_COPPER);
			end
		end
	else
		gsc = "|cffa0a0a0".._G.NONE.."|r";
	end
	
	return gsc;
end

function GFWUtils_temp_ItemLink(linkInfo)
--	local sName, sLink, iQuality, iLevel, _, sType, sSubType, iCount, _, _, _, _, _, _, _, _, _ = GetItemInfo(linkInfo);
	local _, sLink, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(linkInfo);
	return sLink;
end

function GFWUtils_temp_DecomposeItemLink(link)
--	local _, _, _, _, itemID, enchant, gem1, gem2, gem3, gem4, randomProp, uniqueID = string.find(link, "item:(-?%d+):(-?%d+):(-?%d+):(-?%d+):(-?%d+):(-?%d+):(-?%d+):(-?%d+)");
	local _, _, color, itype, itemID, enchant, gem1, gem2, gem3, gem4, suffix, uniqueID, linkLvl, reforging, name = string.find(link, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
	itemID = tonumber(itemID);
	enchant = tonumber(enchant);
	gem1 = tonumber(gem1);
	gem2 = tonumber(gem2);
	gem3 = tonumber(gem3);
	gem4 = tonumber(gem4);
	suffix = tonumber(suffix);
	uniqueID = tonumber(uniqueID);
	return itemID, enchant, gem1, gem2, gem3, gem4, suffix, uniqueID;
end

function GFWUtils_temp_BuildItemLink(itemID, enchant, gem1, gem2, gem3, gem4, suffix, uniqueID)
	itemID = itemID or 0;
	enchant = enchant or 0;
	gem1 = gem1 or 0;
	gem2 = gem2 or 0;
	gem3 = gem3 or 0;
	gem4 = gem4 or 0;
	suffix = suffix or 0;
	uniqueID = uniqueID or 0;
	return string.format("item:%d:%d:%d:%d:%d:%d:%d:%d", itemID, enchant, gem1, gem2, gem3, gem4, suffix, uniqueID);
end

function GFWUtils_temp_ColorToCode(color)
	return string.format("|cff%02x%02x%02x", (color.r * 255), (color.g * 255), (color.b * 255));
end

function GFWUtils_temp_ColorText(text, color)
	return GFWUtils.ColorToCode(color)..text..FONT_COLOR_CODE_CLOSE;
end

------------------------------------------------------
-- load only if not already loaded
------------------------------------------------------

if (GFWUtils == nil) then
	GFWUtils = {};
end
local G = GFWUtils;
if (G.Version == nil or (tonumber(G.Version) ~= nil and G.Version < GFWUTILS_THIS_VERSION)) then

	-- Constants
	GFW_FONT_COLOR = {r=0.25, g=1.0, b=1.0};
	GFW_DEBUG_COLOR = {r=1.0, g=0.75, b=0.25};
	G.Debug = false;

	-- Functions
	G.Hilite = GFWUtils_temp_HiliteText;
	G.Red = GFWUtils_temp_RedText;
	G.Gray = GFWUtils_temp_GrayText;
	G.LtY = GFWUtils_temp_LightYellowText;
	G.Print = GFWUtils_temp_Print;
	G.PrintOnce = GFWUtils_temp_PrintOnce;
	G.DebugLog = GFWUtils_temp_DebugLog;
	G.Note = GFWUtils_temp_Note;
	G.FormatToPattern = GFWUtils_temp_FormatToPattern;
	G.Split = GFWUtils_temp_Split;
	G.TitleCase = GFWUtils_temp_TitleCase;
	G.GSC = GFWUtils_temp_GSC;
	G.TextGSC = GFWUtils_temp_TextGSC;
	G.ItemLink = GFWUtils_temp_ItemLink;
	G.DecomposeItemLink = GFWUtils_temp_DecomposeItemLink;
	G.BuildItemLink = GFWUtils_temp_BuildItemLink;
	G.ColorToCode = GFWUtils_temp_ColorToCode;
	G.ColorText = GFWUtils_temp_ColorText;
	
	G.PlaySound = GFWUtils_temp_PlaySound;
	
	-- Set version number
	G.Version = GFWUTILS_THIS_VERSION;
end

