------------------------------------------------------
-- Main.lua
------------------------------------------------------
local addon, _ = ...
local version = GetAddOnMetadata(addon, "Version");

FT_NewItemInfo = {};
FT_NewVendorInfo = {};
FT_NewVendorLocation = {};

local defaultsDB = {
	NoRecipes = false,
	NoShowCost = false,
	NoDarkmoon = false,
	CoinFormat = "icon", --< icon, short, long
	NoSound = false,
};

function initiateDB(a,b)
	if type(a) ~= "table" then return {} end
	if type(b) ~= "table" then b = {} end
	for k, v in pairs(a) do
		if type(v) == "table" then
			b[k] = initDB(v, b[k])
		elseif type(b[k]) ~= type(v) then
			b[k] = v
		end
	end
	return b
end;

local FT_VersionFrame = CreateFrame("FRAME");
FT_VersionFrame:RegisterEvent("ADDON_LOADED");
function FT_VersionFrame:OnEvent(event, arg1)
	if event == "ADDON_LOADED" and arg1 == addon then
		FleecingTipDB = initiateDB(defaultsDB, FleecingTipDB)
		if FleecingTipDB.Version == nil then
			wipe(FT_NewItemInfo);
			wipe(FT_NewVendorInfo);
			wipe(FT_NewVendorLocation);
			FleecingTipDB.Version = version;
		elseif FleecingTipDB.Version ~= version then
			wipe(FT_NewItemInfo);
			wipe(FT_NewVendorInfo);
			wipe(FT_NewVendorLocation);
			FleecingTipDB.Version = version;
		end
		FT_VersionFrame:UnregisterEvent("ADDON_LOADED");
		GFWUtils.PrintOnce(addon.." "..GFWUtils.LtY(version)..": addon Loaded.", 60);
	end
end
FT_VersionFrame:SetScript("OnEvent", FT_VersionFrame.OnEvent);

function FT_HookTooltip(frame)
	if (frame:GetScript("OnTooltipSetItem")) then
		frame:HookScript("OnTooltipSetItem", FT_OnTooltipSetItem);
	else
		frame:SetScript("OnTooltipSetItem", FT_OnTooltipSetItem);
	end
end




-- return the real zone name for BC or WoD...
function FT_RealMapName(mID)
	if (not mID) then return nil; end;
	local mapDetails = C_Map.GetMapInfo(mID);
	local mapName = mapDetails.name;
	if (mID == 104) or (mID == 107) then
		local mapZone = C_Map.GetMapInfo(466) -- Outland
		mapName = mapName.." ("..mapZone.name..")";
	elseif (mID == 539) or (mID == 550) then
		local mapZone = C_Map.GetMapInfo(572) -- Draenor
		mapName = mapName.." ("..mapZone.name..")";
	end
	return mapName;
end

function FT_GetVendorCoordinates(faction,npc)
	if (not faction) or (not npc) then return nil; end;
--	print("faction="..tostring(faction))
--	print("npc="..tostring(npc))
	local tex = nil;
	local vendorCoords = FT_VendorLocations[faction][npc]
	if ( vendorCoords ~= nil) then
		local uiD = vendorCoords[1];
		if (not uiD) then return nil; end;
		local a1,c1 = "", "";
		if (uiD > 0) then
			a1 = FT_RealMapName(uiD);
		end
		local b1 = FT_VendorLocations[faction][npc][2] or "";
		if (string.len(b1) == 4) then b1 = string.format("%.1f|r",tonumber(b1)); end;
--		if (not b1) then b1 = ""; end;
		local d1 = FT_VendorLocations[faction][npc][3] or "";
		if (string.len(d1) == 4) then d1 = string.format("%.1f|r",tonumber(d1)); end;
--		if (not d1) then d1 = ""; end;
		if (FT_VendorLocations[faction][npc][3]) then c1 = ", "; end;
		if (FT_VendorLocations[faction][npc][2]) then
			tex = a1.." ["..b1..c1..d1.."]";
		else
			tex = a1
		end
	end
	return tex
end

function ERROR_ItemInfo(id,npc)
	local iN, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(id) or "";
	GFWUtils.PrintOnce(GFWUtils.Red(addon.." "..version.." error: ").."Can't find any information on item ["..tostring(id).."] / '"..iN.."' :: Vendor:'"..npc.."' ? ".."Please report this on the website.", 60);
end

function ERROR_VendorInformation(npc)
	GFWUtils.PrintOnce(GFWUtils.Red(addon.." "..version.." error: ").."Can't find any information on '"..npc.."' ? ".."Please report this on the website.", 60);
end

function ERROR_VendorLocation(npc)
	GFWUtils.PrintOnce(GFWUtils.Red(addon.." "..version.." error: ").."Can't find location for '"..npc.."' ? ".."Please report this on the website.", 60);
end

--function ERROR_VendorCoordinates(npc)
--	GFWUtils.PrintOnce(GFWUtils.Red(addon.." "..version.." error: ").."'"..npc.."' coordinates coming soon!", 60);
--end

function FT_FindVendorLocation(faction, npc)
	local vendorFound = FT_VendorLocations[faction][npc];
	if (vendorFound == nil) then
		vendorFound = FT_VendorLocations["Neutral"][npc];
--		print("vendorFound="..tostring(vendorFound))
		if (vendorFound) then
			faction = "Neutral"
		else
			return faction, nil
		end
	end
	return faction, vendorFound
end

function FT_OnTooltipSetItem(self)
	if (MerchantFrame:IsVisible()) then
		for buttonIndex = 1, MERCHANT_ITEMS_PER_PAGE do
			local button = _G["MerchantItem"..buttonIndex.."ItemButton"];
			if (self:IsOwned(button)) then return; end
		end
	end

	local name, link = self:GetItem();
	if (not link) then return; end
	-- workaround for OnTooltipSetItem being called twice for recipes
	local _, _, _, _, _, itemType, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(link);
	if (itemType == FT.RECIPE and self.lastLink ~= link) then
		for i = 1, self:NumLines() do
			local line = _G[self:GetName().."TextLeft"..i];
			local text = line:GetText();
			if (text and string.sub(text, 1, 1) == "\n") then 
				self.lastLink = link;
				return; 
			end
		end
	end
	self.lastLink = nil;
	
	if (link) then
		local _, _, itemID  = string.find(link, "item:(%d+)");
		itemID = tonumber(itemID);
--		itemID = 90655 --#TESTING#--
		local itemInfo = FT_ItemInfo[itemID];
		if (not itemInfo) then
			itemInfo = FT_NewItemInfo[itemID];
		end
--		print("itemID="..tostring(itemID))
		if (not FleecingTipDB.NoRecipes and itemInfo) then
			local myFaction = UnitFactionGroup("player");
			local vendors = GFWTable.Merge(FT_VendorInfo[myFaction][itemID], FT_VendorInfo["Neutral"][itemID]);
			if (FT_NewVendorInfo[myFaction] and FT_NewVendorInfo[myFaction][itemID]) then
				vendors = GFWTable.Merge(vendors, FT_NewVendorInfo[myFaction][itemID]);
			end
			if (FT_NewVendorInfo["Neutral"] and FT_NewVendorInfo["Neutral"][itemID]) then
				vendors = GFWTable.Merge(vendors, FT_NewVendorInfo["Neutral"][itemID]);
			end
			
			local color;
			local intro;
			
			local note, we;
			if (itemInfo.note ~= nil or itemInfo.we ~= nil) then
				note = itemInfo.note;
				we = itemInfo.we;
				color = FT.SPECIAL_VENDOR_COLOR;
			else
				color = GFW_FONT_COLOR;
			end
			
			if (not FleecingTipDB.NoShowCost) then
				local priceSummmary;
				if (itemInfo.b and itemInfo.b > 0) then
				 	priceSummmary = GFWUtils.TextGSC(itemInfo.b);
				else
					priceSummmary = "";
				end
				if (itemInfo.h) then
					priceSummmary = priceSummmary .. " + ".. itemInfo.h.. " honor";
				end
				if (itemInfo.a) then
					priceSummmary = priceSummmary .. " + ".. itemInfo.a.. " arena";
				end
				if (itemInfo.i) then
					priceSummmary = priceSummmary .. " + ".. itemInfo.i;
				end
				priceSummmary = string.gsub(priceSummmary, "^ %+ ", "");
				priceSummmary = string.gsub(priceSummmary, " %+ $", "");
				
				if (itemInfo.need) then
					intro = string.format(FT.NEED_FOR, GFWUtils.Hilite(priceSummmary));
				else
					intro = string.format(FT.SOLD_FOR_PRICE_BY, GFWUtils.Hilite(priceSummmary));
				end
			else
				intro = FT.SOLD_BY;
			end
			
			local professionFound = nil;
			if (itemInfo.made) then
--				color = FT.PATTERN_COLOR
--				intro = FT.MADE_BY;
				professionFound = true;
			end
			
			local dropFound = nil;
			if (itemInfo.i == "Drop") then
				color = FT.DROP_COLOR
				intro = FT.DROPPED_BY;
				dropFound = true;
				myFaction = "Drops";
			end
			
			if (itemInfo.i == "Quest") or
				(itemInfo.i == "Fished") then
				intro = FT.OPTAINABLE_BY;
			end
			
			if (itemInfo.i == "Useless") then
				color = FT.USELESS_COLOR
			end
			
			local numVendors = table.getn(vendors)
			if (numVendors > 1) then
				
				self:AddLine(intro..":", color.r, color.g, color.b);
				for i, aVendor in pairs(vendors) do
					local locFaction = myFaction;
					local vendorLoc;
					locFaction, vendorLoc = FT_FindVendorLocation(locFaction,aVendor)
					
					if (vendorLoc ~= nil) or (itemInfo.made and itemInfo.b) or (dropFound) then
						local vendorName = FT[aVendor] or aVendor;
						local vendorNPC = FT_GetVendorCoordinates(locFaction,aVendor)
--						if (itemInfo.pro) then vendorName = itemInfo.pro; end;
						if (dropFound) then
							if (FT_VendorLocations[locFaction][vendorName][4] ~= nil) then
								local spec = FT_VendorLocations[locFaction][vendorName][4]
								vendorName = vendorName.." ("..spec..")"
							end
						end
						if (vendorNPC) ~= nil then
							self:AddLine(FT.INDENT..string.format(FT.VENDOR_LOCATION_FORMAT, vendorName, vendorNPC), color.r, color.g, color.b);
						else
							self:AddLine(FT.INDENT..vendorName, color.r, color.g, color.b);
--							if (not itemInfo.made) then
--								ERROR_VendorCoordinates(aVendor)
--							end
						end
					else
						ERROR_VendorInformation(aVendor)
						if (aVendor ==numVendors) then
							return false;
						end
					end
					
				end
				
			elseif (numVendors == 1) then
				
				local vendorLoc;
				myFaction, vendorLoc = FT_FindVendorLocation(myFaction,vendors[1])
--				print ("myFaction="..tostring(myFaction));
--				print ("vendorLoc="..tostring(vendorLoc));
--				print ("vendors[1]="..tostring(vendors[1]));
				if (vendorLoc ~= nil) or (itemInfo.made and itemInfo.b) or (dropFound) then
					local vendorName = FT[vendors[1]] or vendors[1];
					local vendorNPC = FT_GetVendorCoordinates(myFaction,vendorName)
--					if (itemInfo.pro) then vendorName = itemInfo.pro; end;
--					print ("vendorName="..tostring(vendorName));
					if (dropFound) then
						if (FT_VendorLocations[myFaction][vendorName][4] ~= nil) then
							local spec = FT_VendorLocations[myFaction][vendorName][4]
							vendorName = vendorName.." ("..spec..")"
						end
					end
--					print ("vendorName="..tostring(vendorName));
					if (vendorNPC ~= nil) and (vendorName ~= "Useless") then
						self:AddLine(intro.." "..string.format(FT.VENDOR_LOCATION_FORMAT, vendorName, vendorNPC).." ", color.r, color.g, color.b);
					else
						if (vendorName == "Useless") then
							self:AddLine(FT.ITEM_WARNING, color.r, color.g, color.b);
						else
							self:AddLine(intro.." "..vendorName.." ", color.r, color.g, color.b);
--							if (not itemInfo.made) then
--								ERROR_VendorCoordinates(vendorName)
--							end
						end
					end
					
				else
					if (not itemInfo.made) then
						ERROR_VendorInformation(vendors[1])
						return false;
					end
				end
				
			else
				
				local found = false;
				for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
					if (FT_VendorInfo[faction][itemID]) then
						found = true;
					end
				end
				if not (found) then
					local link, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemID) or "";
					GFWUtils.PrintOnce(GFWUtils.Red(addon.." "..version.." error: ").."'"..link.."'["..itemID.."] is listed but has no vendors.", 60);
					GFWUtils.PrintOnce("Please report this on the website.", 60);
					return false;
				end
				
			end
			
			if (itemInfo.made) then
				local color = FT.PATTERN_COLOR;
				local iName, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemInfo.id) or "";
				
				local iCost = GFWUtils.TextGSC(itemInfo.cost)
				local noNPC = table.getn(itemInfo.who)
				self:AddLine(FT.MADE_BY..string.format(FT.VENDOR_LOCATION_FORMAT, itemInfo.pro, FT.CLASS_PROFESSION), color.r, color.g, color.b);
				if (noNPC == 1) then
					local NPCname = itemInfo.who[1]
					if (NPCname == "Trainer") then
						self:AddLine(string.format(FT.LEARN_AT, iName), color.r, color.g, color.b);
					else
						local vendorLoc;
						myFaction, vendorLoc = FT_FindVendorLocation(myFaction,NPCname)
						if (vendorLoc ~= nil) then
							local NPCcoords = FT_GetVendorCoordinates(myFaction,NPCname)
							if (NPCcoords ~= nil) then
								self:AddLine(FT.BUY..string.format(FT.PATTERN_FROM, iName, iCost)..NPCname..", "..NPCcoords, color.r, color.g, color.b);
							else
								self:AddLine(FT.BUY..string.format(FT.PATTERN_FROM, iName, iCost)..NPCname, color.r, color.g, color.b);
--								ERROR_VendorCoordinates(NPCname)
							end
						else
							ERROR_VendorInformation(NPCname)
						end
					end
				else
					self:AddLine(FT.BUY..string.format(FT.PATTERN_FROM, iName, iCost), color.r, color.g, color.b);
					for listNames = 1, noNPC do
						local locFaction = myFaction
						local NPCname = itemInfo.who[listNames]
						locFaction, vendorLoc = FT_FindVendorLocation(locFaction,NPCname)
						if (vendorLoc ~= nil) then
							local NPCcoords = FT_GetVendorCoordinates(locFaction,NPCname)
							if (NPCcoords ~= nil) then
								self:AddLine(FT.INDENT..string.format(FT.VENDOR_LOCATION_FORMAT, NPCname, NPCcoords), color.r, color.g, color.b);
							else
								self:AddLine(FT.INDENT..NPCname, color.r, color.g, color.b);
--								ERROR_VendorCoordinates(NPCname)
							end
						else
							ERROR_VendorInformation(NPCname)
						end
					end
				end
			end
			
			if (we ~= nil) then
				self:AddLine("("..FT.WORLD_EVENT..we..")", color.r, color.g, color.b);
			elseif (note ~= "") then
				self:AddLine(note, color.r, color.g, color.b);
			end
			
			GFWUtils.PlaySound(itemID, 30)
			return true;
		end
		
		-- FactionFriend also shows tooltips for this stuff; if it's present, let it take over
		if (FFF_Config and not FFF_Config.NoTooltip) then return; end

		local darkmoonInfo = FT_DarkmoonInfo[itemID];
		if (not FleecingTipDB.NoDarkmoon and darkmoonInfo) then
			local color = FT.SPECIAL_VENDOR_COLOR;
			self:AddLine(darkmoonInfo, color.r, color.g, color.b);
			return true;
		end
	
	end
	
end

function FT_OnEvent(self, event, arg1)

-- In Mists of Pandaria, the game freezes if you attempt to call GetMerchantNumItems() when MERCHANT_SHOW or MERCHANT_UPDATE is called.
-- To correct this, a delay is instituted.  I found that even with .01 delay, it still doesn't freeze.  I can live with a .01 delay.
-- I do worry about the impact to performance this code might have, though.  If you know a better method, feel free to let me know.

	local FT_DelayFrame = CreateFrame("FRAME");
	local FT_DelayTotal = 0;
	function FT_DelayFrame:OnUpdate(self, elapsed)
		FT_DelayTotal = FT_DelayTotal + self;
		if FT_DelayTotal >= .01 then
			FT_DelayTotal = 0;
			FT_DelayFrame:SetScript("OnUpdate", nil);
			FT_ScanMerchant();
		end
	end

	if( event == "MERCHANT_SHOW" or event == "MERCHANT_UPDATE" ) then
--		FT_ScanMerchant();		-- This is where the function was called before delay was instituted.
		FT_DelayFrame:SetScript("OnUpdate", FT_DelayFrame.OnUpdate);

	end
end

function FT_OnLoad(self)

	-- Register Slash Commands
	SLASH_FLEECINGTIP1 = "/fleecingtip";
	SLASH_FLEECINGTIP2 = "/ft";
	SlashCmdList["FLEECINGTIP"] = function(msg)
		FT_ChatCommandHandler(msg);
	end

	
	-- Register for Events
	self:RegisterEvent("MERCHANT_SHOW");
	self:RegisterEvent("MERCHANT_UPDATE");
	
	FT_HookTooltip(GameTooltip);
	FT_HookTooltip(ItemRefTooltip);
	
--	table.insert(UISpecialFrames,"FT_OptionsFrame");	
end

function FT_ChatCommandHandler(msg)
	
	if (msg == "version") then
		GFWUtils.Print(addon.." "..version);
		return;
	end
	
	if ( msg == "sound on" ) or ( msg == "sound off" ) then
		FleecingTipDB.NoSound = not FleecingTipDB.NoSound
		if (FleecingTipDB.NoSound) then
			GFWUtils.Print(addon.." sound is now set to "..GFWUtils.Hilite("off")..".");
		else
			GFWUtils.Print(addon.." sound is now set to "..GFWUtils.Hilite("on")..".");
		end
		return;
	end
	
	if ( msg == "money icon" ) then
		FleecingTipDB.CoinFormat = "icon";
		GFWUtils.Print(addon.." money format is set to "..GFWUtils.Hilite("icon")..".");
		return;
	end
	
	if ( msg == "money short" ) then
		FleecingTipDB.CoinFormat = "short";
		GFWUtils.Print(addon.." money format is set to "..GFWUtils.Hilite("short")..".");
		return;
	end
	
	if ( msg == "money long" ) then
		FleecingTipDB.CoinFormat = "long";
		GFWUtils.Print(addon.." money format is set to "..GFWUtils.Hilite("long")..".");
		return;
	end
	
	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		GFWUtils.Print(addon.." "..version..":");
		GFWUtils.Print("/fleecingtip (or /ft)");
		GFWUtils.Print("- "..GFWUtils.Hilite("help").." - Display this help list.");
		GFWUtils.Print("- "..GFWUtils.Hilite("[item link]").." - Show info for an item in the chat window.");
		GFWUtils.Print("- "..GFWUtils.Hilite("status").." - Check current settings.");
		GFWUtils.Print("- "..GFWUtils.Hilite("money icon").." | "..GFWUtils.Hilite("short").." | "..GFWUtils.Hilite("long").." - Money format display.");
		GFWUtils.Print("- "..GFWUtils.Hilite("sound on").." | "..GFWUtils.Hilite("off").." - Option sound on a found item.");
		GFWUtils.Print("- "..GFWUtils.Hilite("recipes on").." | "..GFWUtils.Hilite("off").." - Show info for vendor-supplied recipes in tooltips.");
		GFWUtils.Print("- "..GFWUtils.Hilite("showcost on").." | "..GFWUtils.Hilite("off").." - Also show vendor prices for recipes.");
		GFWUtils.Print("- "..GFWUtils.Hilite("darkmoon on").." | "..GFWUtils.Hilite("off").." - Show info for Darkmoon Faire grey item turn-ins in tooltips.");
		return;
	end
	
	if ( msg == "status" ) then
		GFWUtils.Print(addon.." status:");
		GFWUtils.Print("Money format is "..GFWUtils.Hilite(FleecingTipDB.CoinFormat)..".");
		if (FleecingTipDB.NoSound) then
			GFWUtils.Print("Sound is "..GFWUtils.Hilite("off")..".");
		else
			GFWUtils.Print("Sound is "..GFWUtils.Hilite("on")..".");
		end
		local gotSomething;
		if (not FleecingTipDB.NoRecipes) then
			GFWUtils.Print("Showing info for "..GFWUtils.Hilite("vendor-supplied recipes").." in tooltips.");
			gotSomething = 1;
			if (not FleecingTipDB.NoShowCost) then
				GFWUtils.Print("Also showing vendor price for recipes.");
			end
		end
		if (not FleecingTipDB.NoDarkmoon) then
			GFWUtils.Print("Showing info for "..GFWUtils.Hilite("Darkmoon Faire grey item turn-ins").." in tooltips.");
			gotSomething = 1;
		end
		if (not gotSomething) then
			GFWUtils.Print("Not adding any info to tooltips.");
		end
		return;
	end
	
	if (msg == "test") then
		local itemInfoCount = 0;
		for itemID in pairs(FT_ItemInfo) do
			local found = false;
			for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
				if (FT_VendorInfo[faction][itemID]) then
					found = true;
				end
			end
			if not (found) then
				GFWUtils.Print("Item ID "..itemID.." not found in FT_VendorInfo.");
			end
			itemInfoCount = itemInfoCount + 1;
		end
		GFWUtils.Print(itemInfoCount.." entries in FT_ItemInfo.");
		for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
			local vendorInfoCount = 0;
			for itemID in pairs(FT_VendorInfo[faction]) do
				if (FT_ItemInfo[itemID] == nil) then
					GFWUtils.Print("Item ID "..itemID.." not found in FT_ItemInfo.");
				end
				vendorInfoCount = vendorInfoCount + 1;
			end
			GFWUtils.Print(vendorInfoCount.." entries in FT_VendorInfo["..faction.."].");
		end
		local dropCount = 0;
		for itemID in pairs(FT_ItemInfo) do
			if (FT_ItemInfo[itemID].i == "Drop") then
				local dropfound = false;
				local dropName = tostring(_G.UNKNOWN);
				local loc = dropName;
				for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
					if (FT_VendorInfo[faction][itemID]) then
						dropName = FT_VendorInfo[faction][itemID][1]
						loc = faction
--						print("Name: "..tostring(dropName)); -- *Debug*
						if ( FT_VendorLocations["Drops"][dropName]) then
							dropfound = true;
						end
					end
				end
				if not (dropfound) then
					GFWUtils.Print("Drop '"..dropName.."' not found for ["..loc.."] "..itemID.." itemID.");
				end
				dropCount = dropCount + 1;
			end
		end
		GFWUtils.Print(dropCount.." 'Drop' entries.");
		return;
	end

	local _, _, cmd, args = string.find(msg, "^([%l%d']+) *(.*)");
	if (cmd) then cmd = string.lower(cmd); end
		
	if (args == nil or args == "") then
		args = msg;
	end
	local postedText;
	for itemLink in string.gmatch(args, "|c%x+|Hitem:[-%d:]+|h%[.-%]|h|r") do
		postedText = nil;
		local _, _, itemID  = string.find(itemLink, "item:(%d+)");
		if (itemID == nil or itemID == "") then
			GFWUtils.Print("Usage: "..GFWUtils.Hilite("/ft info <item link>"));
			return;
		end
		itemID = tonumber(itemID);
		GFWUtils.Print(addon..": found...");
		local itemInfo = FT_ItemInfo[itemID];
		if (itemInfo) then
			local myFaction = UnitFactionGroup("player");
			local vendors = GFWTable.Merge(FT_VendorInfo[myFaction][itemID], FT_VendorInfo["Neutral"][itemID]);
			if (FT_NewVendorInfo[myFaction] and FT_NewVendorInfo[myFaction][itemID]) then
				vendors = GFWTable.Merge(vendors, FT_NewVendorInfo[myFaction][itemID]);
			end
			if (FT_NewVendorInfo["Neutral"] and FT_NewVendorInfo["Neutral"][itemID]) then
				vendors = GFWTable.Merge(vendors, FT_NewVendorInfo["Neutral"][itemID]);
			end
			local note = itemInfo.note;
			local intro = itemLink..": "..string.format(FT.SOLD_FOR_PRICE_BY, GFWUtils.TextGSC(itemInfo.b));

			if (vendors == nil or vendors == {}) then
				GFWUtils.Print(GFWUtils.Red(addon.." "..version.." error: ")..itemLink.."("..itemID..") is listed but has no vendors. Please report this at the website you downloaded from.");
				return;
			end
			GFWUtils.Print(intro);
			for i, aVendor in pairs(vendors) do
				local vendorName = FT[aVendor] or aVendor;
--				local vendorLocation = FT[FT_VendorLocations[myFaction][aVendor]] or FT_VendorLocations[myFaction][aVendor];
--				if (not vendorLocation) then
--					vendorLocation = FT[FT_NewVendorLocation[myFaction][aVendor]] or FT_NewVendorLocation[myFaction][aVendor];
--				end
--				local vendorNPC = FT_GetVendorCoordinates(vendorName)
				local vendorNPC = FT_GetVendorCoordinates(myFaction,aVendor)
--				if (vendorNPC) ~= nil then
					GFWUtils.Print(string.format(FT.VENDOR_LOCATION_FORMAT, vendorName, vendorNPC));
--				else
--					GFWUtils.Print(string.format(FT.VENDOR_LOCATION_FORMAT, vendorName, vendorLocation));
--				end;
			end
			if (note and note ~= "") then
				GFWUtils.Print(GFWUtils.Hilite(note));
			end
			postedText = 1;
		end
		
		local darkmoonInfo = FT_DarkmoonInfo[itemID];
		if (darkmoonInfo) then
			GFWUtils.Print(itemLink..": "..darkmoonInfo);
			postedText = 1;
		end
		
		if (not postedText) then
			GFWUtils.Print("Nothing known about "..itemLink..".");
		end
		return;
	end
	if (postedText) then
		return;
	end
	-- if we made it down here, there were args we didn't understand... time to remind the user what to do.
	FT_ChatCommandHandler("help");

end

function FT_CheckMerchant(itemID)
	for merchantIndex = 1, GetMerchantNumItems() do
		local link = GetMerchantItemLink(merchantIndex);
		local _, _, merchantItemID  = string.find(link, "item:(%d+)");
		if (tonumber(merchantItemID) == itemID) then
			return true;
		end
	end
	return false;
end

function FT_ScanMerchant()
	for index = 1, GetMerchantNumItems() do
		local link = GetMerchantItemLink(index);
		if ( link ) then
			local _, _, _, _, _, itemType, subType, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(link);
			if (not FT_ItemIsBoP(index)) and (not FT_ItemBlacklist(link)) and ((itemType == FT.RECIPE) or (itemType == FT.MISCELLANEOUS and ((subType == FT.PET) or (subType == FT.MOUNT))) or (itemType == FT.CONTAINER and subType == FT.BAG)) then
				local itemID = GFWUtils.DecomposeItemLink(link);
				local added;
				local _, _, price, _, _, _, _, extendedCost = GetMerchantItemInfo(index);
				local priceSummmary;
				if (price and price > 0) then
				 	priceSummmary = GFWUtils.TextGSC(price);
				else
					priceSummmary = "";
				end
				local itemSummary = "";
				if (price == 0 and extendedCost) then
					itemCount = GetMerchantItemCostInfo(index);
					if (itemCount and itemCount > 0) then
						for costIndex = 1, itemCount, 1 do
							local itemTexture, itemValue, itemLink, itemName = GetMerchantItemCostItem(index, costIndex);
							if (not itemName) then
								itemName, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemLink);
								if (not itemName) then
									itemName = "Unknown Item";
								end
							end
							itemSummary = itemSummary .. string.format(" + %d %s", itemValue, itemName);
						end
					end
					priceSummmary = priceSummmary .. itemSummary;
					itemSummary = string.gsub(itemSummary, "^ %+ ", "");
					priceSummmary = string.gsub(priceSummmary, "^ %+ ", "");
					priceSummmary = string.gsub(priceSummmary, " %+ $", "");
					if (itemSummary == "") then
						itemSummary = nil;
					end
				end
				if (not FT_ItemInfo[itemID]) then
--					print("|cffff9900**FT ERROR** link:"..tostring(link).." itemID="..tostring(itemID).." price:"..tostring(price).." info:"..tostring(itemSummary).."|r")
					FT_NewItemInfo[itemID] = { b=price, i=itemSummary, note="" };
					added = true;
				end
				local vendorName = UnitName("npc");
				local vendorFaction = UnitFactionGroup("npc");
				if (vendorFaction ~= UnitFactionGroup("player")) then
					vendorFaction = "Neutral";
				end
				if (FT_VendorInfo[vendorFaction][itemID] == nil or GFWTable.KeyOf(FT_VendorInfo[vendorFaction][itemID], vendorName) == nil) then
					if (FT_NewVendorInfo[vendorFaction] == nil) then
						FT_NewVendorInfo[vendorFaction] = {};
					end
					if (FT_NewVendorInfo[vendorFaction][itemID] == nil) then
						FT_NewVendorInfo[vendorFaction][itemID] = {};
					end
					if (GFWTable.KeyOf(FT_NewVendorInfo[vendorFaction][itemID], vendorName) == nil) then
						table.insert(FT_NewVendorInfo[vendorFaction][itemID], vendorName);
						added = true;
					end
				end
				if (not FT_VendorLocations[vendorFaction][vendorName] and not FT_NewVendorLocation[vendorFaction][vendorName]) then
					if (FT_NewVendorLocation[vendorFaction] == nil) then
						FT_NewVendorLocation[vendorFaction] = {};
					end
					if (FT_NewVendorLocation[vendorFaction][vendorName] == nil) then
						FT_NewVendorLocation[vendorFaction][vendorName] = {};
					end
					FT_NewVendorLocation[vendorFaction][vendorName] = GetRealZoneText();
					added = true;
				end
				if (added) then
					GFWUtils.PrintOnce(string.format(addon.." %s has detected new data!  Please report the following message:  %s [%s] sold for %s by %s (%s) in %s.", version, link, itemID, GFWUtils.Hilite(priceSummmary), GFWUtils.Hilite(vendorName), vendorFaction, GFWUtils.Hilite(GetRealZoneText())));
				end
			end
		end
	end
end

function FT_ItemRequiresFaction(link)
	FT_HiddenTooltip:ClearLines();
	FT_HiddenTooltip:SetHyperlink(link);
	for lineNum = 1, FT_HiddenTooltip:NumLines() do
		local leftText = _G["FT_HiddenTooltipTextLeft"..lineNum]:GetText();
		if (FT_ITEM_REQ_REPUTATION == nil) then
			FT_ITEM_REQ_REPUTATION = GFWUtils.FormatToPattern(ITEM_REQ_REPUTATION);
		end
		local _, _, faction, rep = string.find(leftText, FT_ITEM_REQ_REPUTATION);
		if (faction and rep) then
			return true;
		end
	end
end

-- Attempt #2 at a BoP check, seems to work better =P
-- Version 4.2b adds a BoA check as well.
function FT_ItemIsBoP(index)
	FT_BoPCheckingTooltip:ClearLines();
	FT_BoPCheckingTooltip:SetMerchantItem(index);
	for lineNum=1, FT_BoPCheckingTooltip:NumLines() do
		if lineNum > 4 then
			return false;
		end
		local leftText = _G["FT_BoPCheckingTooltipTextLeft"..lineNum]:GetText();
		if string.find(leftText, ITEM_BIND_ON_PICKUP) then
			return true;
		elseif string.find(leftText, ITEM_BIND_TO_BNETACCOUNT) then
			return true;
		elseif string.find(leftText, ITEM_BIND_TO_ACCOUNT) then
			return true;
		end
	end
end

-- A list of items that we don't want to detect new locations for.
function FT_ItemBlacklist(link)
	local result = false
	local FT_Blacklist = {
	[4496] = true,		-- Small Brown Pouch, sold everywhere.
	[4497] = true,		-- Heavy Brown Bag, sold everywhere.
	[4498] = true,		-- Brown Leather Satchel, sold everywhere.
	[4499] = true,		-- Huge Brown Sack, sold everywhere.
--	[17200] = true,		-- Recipe: Gingerbread Cookie.  Blacklisted because otherwise it would list neutral vendors available in enemy faction cities which serves no purpose (even if you CAN buy them there.)
--	[17201] = true,		-- Recipe: Egg Nog.  Blacklisted because otherwise it would list neutral vendors available in enemy faction cities which serves no purpose (even if you CAN buy them there.)
	[20752] = true,		-- Formula: Minor Mana Oil.  Blacklisted for having a lot of vendors.
	[20753] = true,		-- Formula: Lesser Wizard Oil.  Blacklisted for having a lot of vendors.
	[20758] = true,		-- Formula: Minor Wizard Oil.  Blacklisted for having a lot of vendors.
	[21099] = true,		-- Recipe: Smoked Sagefish.  Blacklisted for having a lot of vendors.
	[21219] = true,		-- Recipe: Sagefish Delight.  Blacklisted for having a lot of vendors.
	[22200] = true,		-- Silver Shafted Arrow.  Blacklisted because it is not a pet but is detected as one.
	[22307] = true,		-- Pattern: Enchanted Mageweave Pouch.  Blacklisted for having a lot of vendors.
	[30744] = true,		-- Draenic Leather Pack, sold everywhere.
	[60335] = true,		-- Thick Hide Pack, sold everywhere.
	[74610] = true,		-- Lunar Lantern.  Blacklisted because although the vendor is neutral, it is only purchaseable by Alliance.
	[74611] = true,		-- Festival Lantern.  Blacklisted because although the vendor is neutral, it is only purchaseable by Horde.
	};
	local itemID = GFWUtils.DecomposeItemLink(link);
	if (FT_Blacklist[itemID]) then
		result = true;
	end;
	return result
end;

-- private, for generating base tables from auto-gathered data
function FT_GetNames()
	for itemID, itemInfo in pairs(FT_NewItemInfo) do
		local name, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemID);
		itemInfo.n = name;
	end
	for faction, vendorList in pairs(FT_NewVendorInfo) do
		for itemID, vendors in pairs(vendorList) do
			if (FT_VendorInfo[faction][itemID]) then
				for _, vendor in pairs(FT_VendorInfo[faction][itemID]) do
					if (not GFWTable.KeyOf(vendors, vendor)) then
						table.insert(vendors, vendor);
					end
				end
			end
			local name, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemID);
			vendors.n = name;
		end
	end
end
